using System;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using ChartDirector;

namespace NetCoreRazorCharts.Pages
{
    public class ZoomscrollpdfModel : PageModel
    {
        //
        // Create a random table for demo purpose.
        //
        private RanTable getRanTable()
        {
            RanTable r = new RanTable(127, 4, 1827);
            r.setDateCol(0, new DateTime(2015, 1, 1), 86400);
            r.setCol(1, 150, -10, 10);
            r.setCol(2, 200, -10, 10);
            r.setCol(3, 250, -8, 8);
            return r;
        }

        //
        // Initialize the WebChartViewer when the page is first loaded
        //
        private void initViewer(RazorChartViewer viewer)
        {
            // The full x-axis range is from Jan 1, 2007 to Jan 1, 2012
            DateTime startDate = new DateTime(2015, 1, 1);
            DateTime endDate = new DateTime(2020, 1, 1);
            viewer.setFullRange("x", startDate, endDate);

            // Initialize the view port to show the last 366 days (out of 1827 days)
            viewer.ViewPortWidth = 366.0 / 1827;
            viewer.ViewPortLeft = 1 - viewer.ViewPortWidth;

            // Set the maximum zoom to 10 days (out of 1827 days)
            viewer.ZoomInWidthLimit = 10.0 / 1827;
        }

        //
        // Draw an XYChart using data from startX to endX
        //
        private XYChart drawXYChart(DateTime startX, DateTime endX)
        {
            // Obtain the random data table
            RanTable r = getRanTable();

            // Select the data for the visible date range startDate to endDate. It is possible there
            // is no data point at exactly startDate or endDate. In this case, we also need the data
            // points that are just outside the visible date range to "overdraw" the line a little
            // bit (the "overdrawn" part will be clipped to the plot area) In this demo, we do this
            // by adding a one day margin to the date range when selecting the data.
            r.selectDate(0, startX.AddDays(-1), endX.AddDays(1));
            // The selected data from the random data table
            DateTime[] timeStamps = Chart.NTime(r.getCol(0));
            double[] dataSeriesA = r.getCol(1);
            double[] dataSeriesB = r.getCol(2);
            double[] dataSeriesC = r.getCol(3);

            //
            // At this stage, we have extracted the visible data. We can use those data to plot the
            // chart.
            //

            //================================================================================
            // Configure overall chart appearance.
            //================================================================================

            // Create an XYChart object of size 640 x 350 pixels
            XYChart c = new XYChart(640, 350);

            // Set the plotarea at (55, 50) with width 80 pixels less than chart width, and height
            // 85 pixels less than chart height. Use a vertical gradient from light blue (f0f6ff) to
            // sky blue (a0c0ff) as background. Set border to transparent and grid lines to white
            // (ffffff).
            c.setPlotArea(55, 50, c.getWidth() - 80, c.getHeight() - 85, c.linearGradientColor(0,
                50, 0, c.getHeight() - 35, 0xf0f6ff, 0xa0c0ff), -1, Chart.Transparent, 0xffffff,
                0xffffff);

            // As the data can lie outside the plotarea in a zoomed chart, we need enable clipping.
            c.setClipping();

            // Add a legend box at (55, 25) using horizontal layout. Use 8pts Arial Bold as font.
            // Set the background and border color to Transparent and use line style legend key.
            LegendBox b = c.addLegend(55, 25, false, "Arial Bold", 8);
            b.setBackground(Chart.Transparent);
            b.setLineStyleKey();

            // Set the axis stem to transparent
            c.xAxis().setColors(Chart.Transparent);
            c.yAxis().setColors(Chart.Transparent);

            // Add axis title using 10pts Arial Bold Italic font
            c.yAxis().setTitle("Ionic Temperature (C)", "Arial Bold Italic", 10);

            //================================================================================
            // Add data to chart
            //================================================================================

            //
            // In this example, we represent the data by lines. You may modify the code below to use
            // other representations (areas, scatter plot, etc).
            //

            // Add a line layer for the lines, using a line width of 2 pixels
            LineLayer layer = c.addLineLayer2();
            layer.setLineWidth(2);

            // In this demo, we do not have too many data points. In real code, the chart may
            // contain a lot of data points when fully zoomed out - much more than the number of
            // horizontal pixels in this plot area. So it is a good idea to use fast line mode.
            layer.setFastLineMode();

            // Now we add the 3 data series to a line layer, using the color red (ff33333), green
            // (008800) and blue (3333cc)
            layer.setXData(timeStamps);
            layer.addDataSet(dataSeriesA, 0xff3333, "Alpha");
            layer.addDataSet(dataSeriesB, 0x008800, "Beta");
            layer.addDataSet(dataSeriesC, 0x3333cc, "Gamma");

            //================================================================================
            // Configure axis scale and labelling
            //================================================================================

            // Set the x-axis scale
            c.xAxis().setDateScale(startX, endX);

            //
            // In this demo, the time range can be from a few years to a few days. We demonstrate
            // how to set up different date/time format based on the time range.
            //

            // If all ticks are yearly aligned, then we use "yyyy" as the label format.
            c.xAxis().setFormatCondition("align", 31104000);
            c.xAxis().setLabelFormat("{value|yyyy}");

            // If all ticks are monthly aligned, then we use "mmm yyyy" in bold font as the first
            // label of a year, and "mmm" for other labels.
            c.xAxis().setFormatCondition("align", 2592000);
            c.xAxis().setMultiFormat(Chart.StartOfYearFilter(), "<*font=bold*>{value|mmm yyyy}",
                Chart.AllPassFilter(), "{value|mmm}");

            // If all ticks are daily algined, then we use "mmm dd<*br*>yyyy" in bold font as the
            // first label of a year, and "mmm dd" in bold font as the first label of a month, and
            // "dd" for other labels.
            c.xAxis().setFormatCondition("align", 86400);
            c.xAxis().setMultiFormat(Chart.StartOfYearFilter(),
                "<*block,halign=left*><*font=bold*>{value|mmm dd<*br*>yyyy}",
                Chart.StartOfMonthFilter(), "<*font=bold*>{value|mmm dd}");
            c.xAxis().setMultiFormat2(Chart.AllPassFilter(), "{value|dd}");

            // For all other cases (sub-daily ticks), use "hh:nn<*br*>mmm dd" for the first label of
            // a day, and "hh:nn" for other labels.
            c.xAxis().setFormatCondition("else");
            c.xAxis().setMultiFormat(Chart.StartOfDayFilter(),
                "<*font=bold*>{value|hh:nn<*br*>mmm dd}", Chart.AllPassFilter(), "{value|hh:nn}");

            return c;
        }

        //
        // Draw the chart
        //
        private void drawChart(RazorChartViewer viewer)
        {
            // Determine the visible x-axis range
            DateTime viewPortStartDate = Chart.NTime(viewer.getValueAtViewPort("x",
                viewer.ViewPortLeft));
            DateTime viewPortEndDate = Chart.NTime(viewer.getValueAtViewPort("x",
                viewer.ViewPortRight));

            // Draw the XYChart
            XYChart c = drawXYChart(viewPortStartDate, viewPortEndDate);

            // Add a title to the chart using 18 pts Times New Roman Bold Italic font
            c.addTitle("   PDF Report Demonstration", "Times New Roman Bold Italic", 18);

            if (viewer.IsAttachmentRequest()) {
                // Output as PNG attachment
                viewer.Image = c.makeWebImage(Chart.PNG);
            } else {
                // Output the chart
                viewer.Image = c.makeWebImage(Chart.SVG);

                // Output Javascript chart model to the browser to suppport tracking cursor
                viewer.ChartModel = c.getJsChartModel();
            }
        }

        //
        // Create a multi-page PDF Report
        //
        private byte[] createPdfReport(RazorChartViewer viewer)
        {
            // The MultiPagePDF object can create PDF from multiple pages, each with one chart
            // object. Since a chart object can contain text (eg. using BaseChart.addText) and other
            // charts (eg. using MultiChart), that means each page can contain text and multiple
            // charts.
            MultiPagePDF doc = new MultiPagePDF();

            // Page configuration - A4 = 210 x 297mm. The PDF default is 96 dpi (dot per inch), so
            // the A4 size is equal to 794 x 1123 dots.
            string pageConfig = "pagewidth = 794; pageHeight = 1123";

            // In this example, we include a cover page with only text. This is by creating an empty
            // pie chart with text only.
            PieChart firstPage = new PieChart(720, 960);
            firstPage.addText(360, 320,
                "<*size=50*>ChartDirector<*br*><*size=30*>PDF Report Demonstration<*/*>",
                "Arial Bold", 30, 0x000000, Chart.Center);
            firstPage.setOutputOptions(pageConfig);
            doc.addPage(firstPage);

            // We include 2 charts per page, with each chart showing one year of data. Each page
            // will also have a header and page number
            int startYear = Chart.NTime(viewer.getValueAtViewPort("x", 0)).Year;
            int endYear = Chart.NTime(viewer.getValueAtViewPort("x", 1)).AddSeconds(-1).Year;
            int pageNumber = 0;

            for(int yyyy = startYear; yyyy <= endYear; yyyy += 2) {
                // This chart is the page.
                MultiChart m = new MultiChart(760, 920);

                // Use addTitle to add a header
                m.addTitle("ChartDirector PDF Report Demonstration", "Arial Bold", 20);

                // Create the first chart
                XYChart c = drawXYChart(new DateTime(yyyy, 1, 1), new DateTime(yyyy + 1, 1, 1));
                m.addChart((m.getWidth() - c.getWidth()) / 2, 100, c);
                c.addTitle("Year " + yyyy);

                XYChart c2 = null;
                if (yyyy < endYear) {
                    // Create the second chart
                    c2 = drawXYChart(new DateTime(yyyy + 1, 1, 1), new DateTime(yyyy + 2, 1, 1));
                    c2.addTitle("Year " + (yyyy + 1));
                    m.addChart((m.getWidth() - c2.getWidth()) / 2, 500, c2);
                }

                // Add the page number
                pageNumber = pageNumber + 1;
                m.addTitle(Chart.BottomCenter, (pageNumber).ToString(), "Arial Bold", 8);

                m.setOutputOptions(pageConfig);
                doc.addPage(m);
            }

            // Output the PDF report
            return doc.outPDF2();
        }

        public IActionResult OnGet()
        {
            RazorChartViewer viewer = new RazorChartViewer(HttpContext, "chart1");
            ViewData["Viewer"] = viewer;

            if (RazorChartViewer.IsPartialUpdateRequest(Request)) {
                // Is a partial update request.
                if (viewer.IsAttachmentRequest()) {
                    if (Request.Query["download"].ToString() == "report") {
                        return File(createPdfReport(viewer), "application/pdf",
                            "chartdirector_report.pdf");
                    } else {
                        drawChart(viewer);
                        return File(viewer.StreamChart(), Response.ContentType,
                            "demo_chart.png");
                    }
                } else {
                    drawChart(viewer);
                    return Content(viewer.PartialUpdateChart());
                }
            }

            //
            // If the code reaches here, it is a full page request.
            //

            // In this exapmle, we just need to initialize the WebChartViewer and draw the chart.
            initViewer(viewer);
            drawChart(viewer);

            return Page();
        }
    }
}

