﻿Imports ChartDirector
Imports System.Collections.Generic
Imports System.Threading.Tasks

Public Class FrmMegaZoomScroll

    ' In this example, we plot 3 data series, each with 10, 0, 0 data points.
    ' So the total Is 30, 0, 0 data points.
    Private Const sampleSize As Integer = 10000000

    ' Data arrays
    Dim timeStamps As Double()
    Dim dataSeriesA As Double()
    Dim dataSeriesB As Double()
    Dim dataSeriesC As Double()

    ' Data Accelerator for handle large data sets
    Dim fastData As DataAccelerator

    ' Will set to true at the end of initialization - prevents events from firing before the
    ' controls are properly initialized.
    Private hasFinishedInitialization As Boolean

    '
    ' From Load Event Handler 
    '
    Private Sub FrmMegaZoomScroll_Load(ByVal sender As Object, ByVal e As EventArgs) _
        Handles MyBase.Load

        ' Load the data
        loadData()

        ' Display initial Message
        Dim c As PieChart = New PieChart(800, 400, &HD0E0FF)
        c.addTitle(Chart.Center, "<*block,halign=left,maxwidth=500*>" &
            "<*font=Arial Bold,size=18,underline=2*>Mega Chart Zoom/Scroll with Track Line<*/font*>" &
            "<*br*><*br*>This example demonstrates a chart with huge amount of data. We limit" &
            "it to 3 lines, each with 10 million points, so that it uses less than 1G of RAM" &
            "(320M to store the data, 600M to plot the data and handle the GUI)." &
            "<*br*><*br*><*br*><*br*>Press the Plot Chart button to plot the chart.")
        winChartViewer1.Image = c.makeImage()

    End Sub

    '
    ' Generate random data
    '
    Private Sub loadData()

        ' To speed up random number generation, we use 3 threads to generate the random data
        ' for the 3 data series. The current thread Is used for generating the timestamps.
        Dim tasks As New List(Of Task)

        tasks.Add(Task.Factory.StartNew(
            Sub()
                dataSeriesA = New RanSeries(109).getSeries2(sampleSize, 2500, -1, 1)
            End Sub
        ))
        tasks.Add(Task.Factory.StartNew(
            Sub()
                dataSeriesB = New RanSeries(110).getSeries2(sampleSize, 2500, -1, 1)
            End Sub
        ))
        tasks.Add(Task.Factory.StartNew(
            Sub()
                dataSeriesC = New RanSeries(111).getSeries2(sampleSize, 2500, -1, 1)
            End Sub
        ))

        timeStamps = New Double(sampleSize) {}
        For i As Integer = 0 To sampleSize - 1
            timeStamps(i) = i
        Next

        Task.WaitAll(tasks.ToArray())

    End Sub

    '
    ' User clicks on the Plot Chart pushbutton
    ' 
    Private Sub plotChartPB_Click(sender As Object, e As EventArgs) Handles plotChartPB.Click

        If hasFinishedInitialization Then
            Exit Sub
        End If

        Dim tasks As New List(Of Task)

        ' Use DataAccerlerate the accelerate rendering. To speed up, we create two threads to
        ' process two of the data series, And use the current thread to process the third series.
        fastData = New DataAccelerator(timeStamps, sampleSize)
        tasks.Add(Task.Factory.StartNew(
            Sub()
                fastData.setDataSeries("mA", dataSeriesA)
            End Sub
        ))
        tasks.Add(Task.Factory.StartNew(
            Sub()
                fastData.setDataSeries("mB", dataSeriesB)
            End Sub
        ))

        fastData.setDataSeries("mC", dataSeriesC)
        Task.WaitAll(tasks.ToArray())

        ' Initialize the WinChartViewer
        initChartViewer(winChartViewer1)
        hasFinishedInitialization = True

        ' Trigger the ViewPortChanged event to draw the chart
        winChartViewer1.updateViewPort(True, True)

    End Sub

    '
    ' Initialize the WinChartViewer
    '
    Private Sub initChartViewer(ByVal viewer As WinChartViewer)

        ' Set the full x range to be the duration of the data
        viewer.setFullRange("x", timeStamps(0), timeStamps(sampleSize - 1))

        ' Initialize the view port to show the latest 20% of the time range
        viewer.ViewPortWidth = 0.2
        viewer.ViewPortLeft = 1 - viewer.ViewPortWidth

        ' Set the maximum zoom to 10 points
        viewer.ZoomInWidthLimit = 10.0 / timeStamps.Length

        ' Enable mouse wheel zooming by setting the zoom ratio to 1.1 per wheel event
        viewer.MouseWheelZoomRatio = 1.1

        ' Initially set the mouse usage to "Pointer" mode (Drag to Scroll mode)
        pointerPB.Checked = True
        viewer.MouseUsage = WinChartMouseUsage.ScrollOnDrag

    End Sub

    '
    ' Pointer (Drag to Scroll) button event handler
    '
    Private Sub pointerPB_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) _
        Handles pointerPB.CheckedChanged

        If sender.Checked Then
            winChartViewer1.MouseUsage = WinChartMouseUsage.ScrollOnDrag
        End If

    End Sub

    '
    ' Zoom In button event handler
    '
    Private Sub zoomInPB_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) _
        Handles zoomInPB.CheckedChanged

        If sender.Checked Then
            winChartViewer1.MouseUsage = WinChartMouseUsage.ZoomIn
        End If

    End Sub

    '
    ' Zoom Out button event handler
    '
    Private Sub zoomOutPB_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) _
        Handles zoomOutPB.CheckedChanged

        If sender.Checked Then
            winChartViewer1.MouseUsage = WinChartMouseUsage.ZoomOut
        End If

    End Sub

    '
    ' The ViewPortChanged event handler. This event occurs if the user scrolls or zooms in
    ' or out the chart by dragging or clicking on the chart. It can also be triggered by
    ' calling WinChartViewer.updateViewPort.
    '
    Private Sub winChartViewer1_ViewPortChanged(ByVal sender As Object,
        ByVal e As WinViewPortEventArgs) Handles winChartViewer1.ViewPortChanged

        If Not hasFinishedInitialization Then
            Exit Sub
        End If

        ' In addition to updating the chart, we may also need to update other controls that
        ' changes based on the view port.
        updateControls(winChartViewer1)

        ' Update the chart if necessary
        If e.NeedUpdateChart Then
            drawChart(winChartViewer1)
        End If

    End Sub

    '
    ' Update controls when the view port changed
    '
    Private Sub updateControls(ByVal viewer As WinChartViewer)

        ' In this demo, we need to update the scroll bar to reflect the view port position and
        ' width of the view port.
        hScrollBar1.Enabled = viewer.ViewPortWidth < 1
        hScrollBar1.LargeChange = Math.Ceiling(viewer.ViewPortWidth *
            (hScrollBar1.Maximum - hScrollBar1.Minimum))
        hScrollBar1.SmallChange = Math.Ceiling(hScrollBar1.LargeChange * 0.1)
        hScrollBar1.Value = Math.Round(viewer.ViewPortLeft *
            (hScrollBar1.Maximum - hScrollBar1.Minimum)) + hScrollBar1.Minimum

    End Sub

    '
    ' The scroll bar event handler
    '
    Private Sub hScrollBar1_ValueChanged(ByVal sender As Object, ByVal e As EventArgs) _
        Handles hScrollBar1.ValueChanged

        ' When the view port is changed (user drags on the chart to scroll), the scroll bar will get
        ' updated. When the scroll bar changes (eg. user drags on the scroll bar), the view port will
        ' get updated. This creates an infinite loop. To avoid this, the scroll bar can update the 
        ' view port only if the view port is not updating the scroll bar.
        If hasFinishedInitialization And Not winChartViewer1.IsInViewPortChangedEvent Then
            ' Set the view port based on the scroll bar
            winChartViewer1.ViewPortLeft = (hScrollBar1.Value - hScrollBar1.Minimum) /
                (hScrollBar1.Maximum - hScrollBar1.Minimum)

            ' Trigger a view port changed event to update the chart
            winChartViewer1.updateViewPort(True, False)
        End If

    End Sub

    '
    ' Draw the chart.
    '
    Private Sub drawChart(ByVal viewer As WinChartViewer)

        ' Get the start date and end date that are visible on the chart.
        Dim viewPortStartDate As Double = viewer.getValueAtViewPort("x", viewer.ViewPortLeft)
        Dim viewPortEndDate As Double = viewer.getValueAtViewPort("x", viewer.ViewPortRight)
        fastData.setSubsetRange(viewPortStartDate, viewPortEndDate)

        '
        ' At this stage, we have extracted the visible data. We can use those data to plot the chart.
        '

        '================================================================================
        ' Configure overall chart appearance.
        '================================================================================

        ' Create an XYChart object of size 800 x 400 pixels
        Dim c As XYChart = New XYChart(800, 400)

        ' Set the plotarea at (0, 0) with width 1 pixel less than chart width, And height 20 pixels
        ' less than chart height. Use a vertical gradient from light blue (f0f6ff) to sky blue (a0c0ff)
        ' as background. Set border to transparent And grid lines to white (ffffff).
        c.setPlotArea(0, 0, c.getWidth() - 1, c.getHeight() - 20, c.linearGradientColor(0, 0, 0,
            c.getHeight() - 20, &HF0F6FF, &HA0C0FF), -1, Chart.Transparent, &HFFFFFF, &HFFFFFF)

        ' As the data can lie outside the plotarea in a zoomed chart, we need enable clipping.
        c.setClipping()

        ' Add a legend box at the right side using horizontal layout. Use 10pt Arial Bold as font. Set
        ' the background And border color to Transparent And use line style legend key.
        Dim b As LegendBox = c.addLegend(c.getWidth() - 1, 10, False, "Arial Bold", 10)
        b.setBackground(Chart.Transparent)
        b.setAlignment(Chart.Right)
        b.setLineStyleKey()

        ' Set the x And y axis stems to transparent And the label font to 10pt Arial
        c.xAxis().setColors(Chart.Transparent)
        c.yAxis().setColors(Chart.Transparent)
        c.xAxis().setLabelStyle("Arial", 10)
        c.yAxis().setLabelStyle("Arial", 10, &H336699)

        ' Configure the y-axis label to be inside the plot area And above the horizontal grid lines
        c.yAxis().setLabelGap(-1)
        c.yAxis().setMargin(20)
        c.yAxis().setLabelAlignment(1)

        ' Configure the x-axis labels to be to the left of the vertical grid lines
        c.xAxis().setLabelAlignment(1)

        '================================================================================
        ' Add data to chart
        '================================================================================

        ' Add line layers using the DataAccelerator. Each layer only supports one accelerated
        ' series, so we add 3 layers for the 3 data series.

        ' Add a line layer for the lines, using a line width of 2 pixels
        Dim layer As LineLayer = c.addLineLayer(fastData, "mA", &HFF0000, "Alpha")
        layer.setLineWidth(2)

        Dim layer2 As LineLayer = c.addLineLayer(fastData, "mB", &HCC00, "Beta")
        layer2.setLineWidth(2)

        Dim layer3 As LineLayer = c.addLineLayer(fastData, "mC", &HFF, "Gamma")
        layer3.setLineWidth(2)

        '================================================================================
        ' Configure axis scale and labelling
        '================================================================================

        ' Set the x-axis as a date/time axis with the scale according to the view port x range.
        viewer.syncLinearAxisWithViewPort("x", c.xAxis())

        ' For the automatic axis labels, set the minimum spacing to 75/40 pixels for the x/y axis.
        c.xAxis().setTickDensity(75)
        c.yAxis().setTickDensity(40)

        ' Set the auto-scale margin to 0.05, And the zero affinity to 0.2
        c.yAxis().setAutoScale(0.05, 0.05, 0.2)

        '================================================================================
        ' Output the chart
        '================================================================================

        ' We need to update the track line too. If the mouse Is moving on the chart (eg. if 
        ' the user drags the mouse on the chart to scroll it), the track line will be updated
        ' in the MouseMovePlotArea event. Otherwise, we need to update the track line here.
        If (Not viewer.IsInMouseMoveEvent) AndAlso viewer.IsMouseOnPlotArea Then
            trackLineLabel(c, viewer.PlotAreaMouseX)
        End If

        viewer.Chart = c

    End Sub

    '
    ' Draw track cursor when mouse is moving over plotarea
    '
    Private Sub winChartViewer1_MouseMovePlotArea(ByVal sender As Object,
        ByVal e As System.Windows.Forms.MouseEventArgs) Handles winChartViewer1.MouseMovePlotArea

        If Not hasFinishedInitialization Then
            Exit Sub
        End If

        Dim viewer As WinChartViewer = sender
        trackLineLabel(viewer.Chart, viewer.PlotAreaMouseX)
        viewer.updateDisplay()

        ' Hide the track cursor when the mouse leaves the plot area
        viewer.removeDynamicLayer("MouseLeavePlotArea")

    End Sub

    '
    ' Draw track line with data labels
    '
    Private Sub trackLineLabel(ByVal c As XYChart, ByVal mouseX As Integer)

        ' Clear the current dynamic layer and get the DrawArea object to draw on it.
        Dim d As DrawArea = c.initDynamicLayer()

        ' The plot area object
        Dim plotArea As PlotArea = c.getPlotArea()

        ' Get the data x-value that is nearest to the mouse, and find its pixel coordinate.
        Dim xValue As Double = c.getNearestXValue(mouseX)
        Dim xCoor As Integer = c.getXCoor(xValue)

        ' Draw a vertical track line at the x-position
        d.vline(plotArea.getTopY(), plotArea.getBottomY(), xCoor, d.dashLineColor(&H0, &H101))

        ' Draw a label on the x-axis to show the track line position.
        Dim xlabel As String = "<*font,bgColor=000000*> " & xValue & " <*/font*>"
        Dim t As TTFText = d.text(xlabel, "Arial Bold", 10)

        ' Restrict the x-pixel position of the label to make sure it stays inside the chart image.
        Dim xLabelPos As Integer = Math.Max(0, Math.Min(xCoor - t.getWidth() / 2, c.getWidth() - t.getWidth(
            )))
        t.draw(xLabelPos, plotArea.getBottomY() + 2, &HFFFFFF)

        ' Iterate through all layers to draw the data labels
        For i As Integer = 0 To c.getLayerCount() - 1
            Dim layer As Layer = c.getLayerByZ(i)

            ' The data array index of the x-value
            Dim xIndex As Integer = layer.getXIndexOf(xValue)

            ' Iterate through all the data sets in the layer
            For j As Integer = 0 To layer.getDataSetCount() - 1
                Dim dataSet As ChartDirector.DataSet = layer.getDataSetByZ(j)

                ' Get the color and position of the data label
                Dim color As Integer = dataSet.getDataColor()
                Dim yCoor As Integer = c.getYCoor(dataSet.getPosition(xIndex), dataSet.getUseYAxis())

                ' Draw a track dot with a label next to it for visible data points in the plot area
                If (yCoor >= plotArea.getTopY()) And (yCoor <= plotArea.getBottomY()) And (color <>
                    Chart.Transparent) And (Not String.IsNullOrEmpty(dataSet.getDataName())) Then

                    d.circle(xCoor, yCoor, 4, 4, color, color)

                    Dim label As String = "<*font,bgColor=" & Hex(color) & "*> " & c.formatValue(
                        dataSet.getValue(xIndex), "{value|P4}") & " <*/font*>"
                    t = d.text(label, "Arial Bold", 10)

                    ' Draw the label on the right side of the dot if the mouse is on the left side the chart,
                    ' and vice versa. This ensures the label will not go outside the chart image.
                    If xCoor <= (plotArea.getLeftX() + plotArea.getRightX()) / 2 Then
                        t.draw(xCoor + 5, yCoor, &HFFFFFF, Chart.Left)
                    Else
                        t.draw(xCoor - 5, yCoor, &HFFFFFF, Chart.Right)
                    End If
                End If
            Next
        Next

    End Sub

End Class